/*
 * Try to be a little smarter about which kernel are we currently running
 */

#ifndef __rh_config_h__
#define __rh_config_h__

/*
 * First, get the version string for the running kernel from
 * /boot/kernel.h - initscripts should create it for us
 */

#include "/boot/kernel.h"

#if defined(__BOOT_KERNEL_DEFAULT) && (__BOOT_KERNEL_DEFAULT == 1)
#define __module__default
#endif

#if defined(__BOOT_KERNEL_SMP) && (__BOOT_KERNEL_SMP == 1)
#define __module__smp
#endif

#if defined(__BOOT_KERNEL_XEN0) && (__BOOT_KERNEL_XEN0 == 1)
#define __module__xen0
#endif

#if defined(__BOOT_KERNEL_LEGACY) && (__BOOT_KERNEL_LEGACY == 1)
#define __module__legacy
#endif

#if defined(__BOOT_KERNEL_ENTERPRISE) && (__BOOT_KERNEL_ENTERPRISE == 1)
#define __module__enterprise
#endif

/* Characterize SMP capabilities of the "default" kernel */
#ifdef __module__default
# if defined(__i386__) || defined(__x86_64__) || defined(__ia64__)
#  define __module__smp
# endif
#endif
/* ... at this point, __module__default shall not be used but to
   define UTS_RELEASE */

#if defined(__module__legacy) || defined(__module__enterprise) || defined(__module__xen0)
# define __module__smp
#else
# define __module__up
#endif

#ifdef __i386__
# ifdef __MODULE_KERNEL_i586
#  define __module__i586
#  ifdef __module__up
#   define __module__i586_up
#  endif
#  ifdef __module__smp
#   define __module__i586_smp
#  endif
#  ifdef __module__legacy
#   define __module__i586_legacy
#  endif
# elif defined(__MODULE_KERNEL_i686)
#  define __module__i686
#  ifdef __module__up
#   define __module__i686_up
#  endif
#  ifdef __module__smp
#   define __module__i686_smp
#  endif
#  ifdef __module__xen0
#   define __module__i686_xen0
#  endif
#  ifdef __module__legacy
#   define __module__i686_legacy
#  endif
#  ifdef __module__enterprise
#   define __module__i686_enterprise
#  endif
# else
#  define __module__i386
#  ifdef __module__up
#   define __module__i386_up
#  endif
#  ifdef __module__smp
#   define __module__i386_smp
#  endif
#  ifdef __module__xen0
#   define __module__i386_xen0
#  endif
#  ifdef __module__legacy
#   define __module__i386_legacy
#  endif
#  ifdef __module__enterprise
#   define __module__i386_enterprise
#  endif
# endif
#endif

#ifdef __x86_64__
# define __module__x86_64
# ifdef __module__up
#  define __module__x86_64_up
# endif
# ifdef __module__smp
#  define __module__x86_64_smp
# endif
# ifdef __module__xen0
#  define __module__x86_64_xen0
# endif
#endif

#ifdef __ia64__
# define __module__ia64
# ifdef __module__smp
#  define __module__ia64_smp
# endif
# ifdef __module__xen0
#  define __module__ia64_xen0
# endif
#endif

#ifdef __powerpc__
# define __module__ppc
# ifdef __module__up
#  define __module__ppc_up
# endif
# ifdef __module__smp
#  define __module__ppc_smp
# endif
#endif

#ifdef __alpha__
# define __module__alpha
# ifdef __module__up
#  define __module__alpha_up
# endif
# ifdef __module__smp
#  define __module__alpha_smp
# endif
#endif

#ifdef __sparc__
# define __module__sparc
# ifdef __module__up
#  define __module__sparc_up
# endif
# ifdef __module__smp
#  define __module__sparc_smp
# endif
#endif

#if defined(__module__smp)
#define _ver_str(x) smp_ ## x
#else
#define _ver_str(x) x
#endif

#endif /* __rh_config_h__ */
