/*************************************************** */
/* Rule Set Based Access Control                     */
/* Author and (c) 1999-2005: Amon Ott <ao@rsbac.org> */
/* Memory allocation                                 */
/* Last modified: 08/Jun/2005                        */
/*************************************************** */

#ifndef __RSBAC_RKMEM_H
#define __RSBAC_RKMEM_H

#include <linux/init.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/timer.h>

#ifdef CONFIG_RSBAC_INIT_DELAY
void rsbac_kmem_cache_sizes_init(void);
#else
void __init rsbac_kmem_cache_sizes_init(void);
#endif

/* if you might need more than this in 2.4, use rsbac_vkmalloc! */
/* In 2.6, this is the maximum to allocate, more will fail! */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

#ifdef CONFIG_MMU
#define RSBAC_MAX_KMALLOC 131072
#else
#ifndef CONFIG_LARGE_ALLOCS
#define RSBAC_MAX_KMALLOC 1048576
#else
#define RSBAC_MAX_KMALLOC 33554432
#endif
#endif

#define rsbac_vmalloc(x) kmalloc(x, GFP_ATOMIC)
#define rsbac_vfree(x) kfree(x)

/* < 2.6 */
#else
#define RSBAC_MAX_KMALLOC 8192
#define rsbac_vmalloc(x) vmalloc(x)
#define rsbac_vfree(x) vfree(x)
#endif

struct rsbac_delayed_kfree_t
  {
    struct timer_list timer;
    void * objp;
  };

extern void * rsbac_kmalloc (size_t size);

/* Allocate memory. This function decides which alloc type (kmalloc or vmalloc) */
/* to use and returns the decision as boolean value (TRUE = vmalloc), */
/* if vmalloc_used_p is not NULL (what it should rather be...) */
extern void * rsbac_vkmalloc (size_t size, rsbac_boolean_t * vmalloc_used_p);

extern void rsbac_kfree (const void *objp);

extern void rsbac_delayed_kfree(void *objp, rsbac_time_t delay);

/* Free the allocated mem. Needs old returned mem type value, */
/* or FALSE, if no type stored (kmalloc type assumed) */
extern void rsbac_vkfree (void *objp, rsbac_boolean_t vmalloc_used);

#endif
